/*
Copyright (C) 2016 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of simcaq-node.

simcaq-node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

simcaq-node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with simcaq-node.  If not, see <https://www.gnu.org/licenses/>.
*/

const express = require('express');

const simcaqFirstReportApp = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const reqBody = require(`${libs}/middlewares/reqBody`);

const config = require(`${libs}/config`); 

const cache = require('apicache').options({ debug: config.debug, statusCodes: {include: [200]} }).middleware;

let rqf = new ReqQueryFields();

const id2str = require(`${libs}/middlewares/id2str`);

simcaqFirstReportApp.use(cache('15 day'));

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addValue({
    name: 'year',
    table: 'simcaq_relatorio_1',
    tableField: 'ano_censo',
    where: {
        relation: '=',
        type: 'integer',
        field: 'ano_censo'
    }
}).addValue({
    name: 'adm_dependency',
    table: 'simcaq_relatorio_1',
    tableField: 'dependencia_adm_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'dependencia_adm_id'
    }
}).addValue({
    name: 'city',
    table: 'simcaq_relatorio_1',
    tableField: 'municipio_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'municipio_id'
    }
}).addValue({
    name: 'state',
    table: 'simcaq_relatorio_1',
    tableField: 'estado_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'estado_id'
    }
}).addValue({
    name: 'school',
    table: 'simcaq_relatorio_1',
    tableField: 'escola_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'escola_id'
    }
}).addValue({
    name: 'location',
    table: 'simcaq_relatorio_1',
    tableField: 'localizacao_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'localizacao_id'
    }
}).addValue({
    name: 'education_level_short_id',
    table: 'simcaq_relatorio_1',
    tableField: 'etapa',
    where: {
        relation: '=',
        type: 'integer',
        field: 'etapa'
    }
}).addValue({
    name: 'shift_id',
    table: 'simcaq_relatorio_1',
    tableField: 'turno',
    where: {
        relation: '=',
        type: 'integer',
        field: 'turno'
    }
});

simcaqFirstReportApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    req.sql.from('simcaq_relatorio_1')
        .field('simcaq_relatorio_1.etapa', 'education_level_short_id')
        .field('simcaq_relatorio_1.turno', 'shift_id')
        .field('simcaq_relatorio_1.localizacao_id', 'location')
        .field('SUM(simcaq_relatorio_1.num_matriculas)', 'num_enrollments')
        .field('SUM(simcaq_relatorio_1.num_escolas)', 'num_schools')
        .field('ano_censo', 'year')
        .group('ano_censo')
        .group('simcaq_relatorio_1.etapa')
        .group('simcaq_relatorio_1.turno')
        .group('simcaq_relatorio_1.localizacao_id');
    next();
}, query, (req, res, next) => {
    req.result.forEach((result) => {
        result.shift = id2str.shift(result.shift_id);
    });
    next();
}, response('simcaqFirstReport'));

module.exports = simcaqFirstReportApp;
