/*
Copyright (C) 2016 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of simcaq-node.

simcaq-node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

simcaq-node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with simcaq-node.  If not, see <https://www.gnu.org/licenses/>.
*/

const express = require('express');

const simcaqSecondReportApp = express.Router();

const libs = `${process.cwd()}/libs`;

const squel = require('squel');

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const reqBody = require(`${libs}/middlewares/reqBody`);

const config = require(`${libs}/config`); 

const cache = require('apicache').options({ debug: config.debug, statusCodes: {include: [200]} }).middleware;

let rqf = new ReqQueryFields();

simcaqSecondReportApp.use(cache('15 day'));

rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addValue({
    name: 'year',
    table: 'simcaq_relatorio_2',
    tableField: 'ano_censo',
    where : {
        relation: '=',
        type: 'integer',
        field: 'ano_censo'
    }
}).addValue({
    name: 'school',
    table: 'simcaq_relatorio_2',
    tableField: 'escola_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'escola_id'
    }
}).addValue({
    name: 'city',
    table:'simcaq_relatorio_2',
    tableField: 'municipio_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'municipio_id'
    }
}).addValue({
    name: 'adm_dependency',
    table: 'simcaq_relatorio_2',
    tableField: 'dependencia_adm_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'dependencia_adm_id'
    }
}).addValue({
    name: 'state',
    table: 'simcaq_relatorio_2',
    tableField: 'estado_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'estado_id'
    }
});

simcaqSecondReportApp.get('/', rqf.parse(), rqf.build(), (req, res, next) => {
    req.sql.from('simcaq_relatorio_2')
        .field('sum(num_matriculas_total)' ,'num_enrollments_total')
        .field('sum(num_matriculas_rural)' ,'num_enrollments_rural')
        .field('sum(num_matriculas_urbana)' ,'num_enrollments_urban')
        .field('sum(num_escolas_total)' ,'num_schools_total')
        .field('sum(num_escolas_rural)' ,'num_schools_rural')
        .field('sum(num_escolas_urbana)' ,'num_schools_urban')
        .field('dependencia_adm_priv', 'private_adm_dependency')
        .field('ano_censo', 'year')
        .group('ano_censo')
        .group('dependencia_adm_priv');
    next();
}, query, response('simcaqSecondReport'));

module.exports = simcaqSecondReportApp;
